@@ -2,7 +2,7 @@
 /*! \file
  * \brief Configuration handling
  *
- * Copyright (c) 2012-2021 by the developers. See the LICENSE file for details.
+ * Copyright (c) 2012-2023 by the developers. See the LICENSE file for details.
  *
  * If nothing else is specified, functions return zero to indicate success
  * and a negative value to indicate an error.
@@ -375,6 +375,25 @@ static void  conf_init_configuration(struct conf*  cfg)
                             empty);
    }
 
+   /* Group list refresh interval */
+   conf_add_entry_int(cfg, CONF_REFRESH_INTERVAL, "refresh_interval:", 0);
+
+   /* Skip to next group for unread article */
+   conf_add_entry_int(cfg, CONF_UNREAD_IN_NEXT_GROUP, "unread_in_next_group:",
+                       0);
+
+   /* Color for signature */
+   conf_add_entry_int(cfg, CONF_COLOR_SIGNATURE, "color_signature:", 43);
+
+   /* Color for external citation */
+   conf_add_entry_int(cfg, CONF_COLOR_EXTERNAL, "color_external:", 88);
+
+   /* Colors of thread citation level */
+   conf_add_entry_int(cfg, CONF_COLOR_LEVEL1, "color_level1:", 140);
+   conf_add_entry_int(cfg, CONF_COLOR_LEVEL2, "color_level2:", 152);
+   conf_add_entry_int(cfg, CONF_COLOR_LEVEL3, "color_level3:", 91);
+   conf_add_entry_int(cfg, CONF_COLOR_LEVEL4, "color_level4:", 75);
+
    /* On success, store flag that configuration was initialized */
    if(error)  { PRINT_ERROR("Initializing configuration failed"); }
    else  { conf_initialized = 1; }
@@ -716,9 +735,13 @@ void  conf_delete(struct conf*  cfg)
          {
             secure_clear_string(config[i].val.s);
             posix_free((void*) cfg[i].val.s);
+            cfg[i].val.s = NULL;
          }
       }
    }
+
+   /* Clear flag that configuration is initialized */
+   conf_initialized = 0;
 }
 
 
@@ -728,7 +751,7 @@ void  conf_delete(struct conf*  cfg)
  * \param[in] cfg  Pointer to configuration array
  *
  * For configuration entries with string type, this function allocates memory.
- * The caller is responsible to free this memory with the function
+ * On success the caller is responsible to free this memory with the function
  * \ref conf_delete() .
  *
  * \return
@@ -773,6 +796,9 @@ int  conf_load(struct conf*  cfg)
 
       /* Release memory for pathname */
       posix_free((void*) confpathname);
+
+      /* Delete configuration again on error */
+      if(res)  { conf_delete(cfg); }
    }
 
    return(res);
@@ -837,17 +863,17 @@ int  conf_store(struct conf*  cfg)
       res = conf_export_configuration(cfg, fs, fs_tmp);
    }
 
-   /* Flush stream of temporary file*/
-   if (!res)  { fu_sync(fd_tmp, fs_tmp); }
+   /* Flush stream of temporary file */
+   if(!res)  { res = fu_sync(fd_tmp, fs_tmp); }
 
    /* Rename temporary file to config file */
    if(!res)  { res = posix_rename(tmppathname, confpathname); }
 
-   /* Unlink temporary file after error */
+   /* Try to unlink temporary file after error */
    if(res)
    {
       PRINT_ERROR("Failed to store configuration in config file");
-      if(tmppathname)  { fu_unlink_file(tmppathname); }
+      if(tmppathname)  { (void) fu_unlink_file(tmppathname); }
    }
 
    /* The tmp file must be removed before releasing the config file lock! */
@@ -889,7 +915,7 @@ int  conf_string_replace(struct conf*  cfg, const char*  s)
    size_t  len;
    char*  p;
 
-   if(NULL != cfg && NULL != s)
+   if(NULL != cfg && NULL != cfg->val.s && NULL != s)
    {
       if(CONF_TYPE_STRING == cfg->type)
       {
@@ -897,7 +923,7 @@ int  conf_string_replace(struct conf*  cfg, const char*  s)
          p = posix_malloc(++len);
          if(NULL != p)
          {
-            strcpy(p , s);
+            strcpy(p, s);
             secure_clear_string(cfg->val.s);
             posix_free((void*) cfg->val.s);
             cfg->val.s = p;
@@ -910,6 +936,41 @@ int  conf_string_replace(struct conf*  cfg, const char*  s)
 }
 
 
+/* ========================================================================== */
+/*! \brief Check integer value against lower and upper bounds
+ *
+ * \param[in] id   ID of integer value to check
+ * \param[in] min  Lower limit
+ * \param[in] max  Upper limit
+ *
+ * \attention The parameter \e id must be associated with an integer value!
+ *
+ * The integer value associated with \e id is compared against the specified
+ * limits.
+ * <br>
+ * If the value is lower than \e min or greater than \e max , it is clamped to
+ * the limits. Otherwise the value is unchanged.
+ *
+ * \return
+ * - New value
+ */
+
+int  conf_integer_check(enum conf_entry  id, int  min, int max)
+{
+   if(min > config[id].val.i)
+   {
+      config[id].val.i = min;
+   }
+
+   if(max < config[id].val.i)
+   {
+      config[id].val.i = max;
+   }
+
+   return(config[id].val.i);
+}
+
+
 /*! @} */
 
 /* EOF */
