@@ -520,13 +520,16 @@ int  db_add(const char*  group, core_anum_t  anum,
                      rv = fu_write_to_filedesc(fd, header, len);
                      if(!rv)
                      {
-                        fu_sync(fd, NULL);
-                        /* Rename temporary file to new entry file */
-                        strcpy(pathname, db_path);
-                        strcat(pathname, group);
-                        strcat(pathname, "/");
-                        strcat(pathname, file);
-                        rv = posix_rename(tmppathname, pathname);
+                        rv = fu_sync(fd, NULL);
+                        if(!rv)
+                        {
+                           /* Rename temporary file to new entry file */
+                           strcpy(pathname, db_path);
+                           strcat(pathname, group);
+                           strcat(pathname, "/");
+                           strcat(pathname, file);
+                           rv = posix_rename(tmppathname, pathname);
+                        }
                      }
                      if(rv)
                      {
@@ -534,7 +537,7 @@ int  db_add(const char*  group, core_anum_t  anum,
                         PRINT_ERROR("Failed to store data");
                         if(NULL != tmppathname)
                         {
-                           fu_unlink_file(tmppathname);
+                           (void) fu_unlink_file(tmppathname);
                         }
                      }
                      else  { res = 0; }
@@ -739,7 +742,7 @@ int  db_delete(const char*  group, core_anum_t  start, core_anum_t  end)
                      pathname[path_len] = 0;
                      strncpy(&pathname[path_len], file, 17);
                      /* Unlink file of entry */
-                     fu_unlink_file(pathname);
+                     (void) fu_unlink_file(pathname);
                      /* Continue if entry was not found */
                   }
                   if(i == end)  { res = 0; }
