@@ -2,7 +2,7 @@
 /*! \file
  * \brief External program delegation functions
  *
- * Copyright (c) 2012-2020 by the developers. See the LICENSE file for details.
+ * Copyright (c) 2012-2022 by the developers. See the LICENSE file for details.
  *
  * If nothing else is specified, functions return zero to indicate success
  * and a negative value to indicate an error.
@@ -365,7 +365,12 @@ int  ext_handler_email(const char*  recipient, const char*  subject,
 /* ========================================================================== */
 /*! \brief Start external handler for URI
  *
- * \param[in] uri  Pointer to URI string
+ * \param[in]  uri      Pointer to URI string
+ * \param[out] invalid  Pointer to invalid encoding flag
+ *
+ * If the URI encoding of \e uri is invalid, a negative value is returned and
+ * a nonzero value is written to the location pointed to by \e invalid .
+ * Otherwise zero is written to the location pointed to by \e invalid .
  *
  * This function calls the external program \c xdg-open to handle the URI.
  *
@@ -383,7 +388,7 @@ int  ext_handler_email(const char*  recipient, const char*  subject,
  * - Positive value if shell reported an error
  */
 
-int  ext_handler_uri(const char*  uri)
+int  ext_handler_uri(const char*  uri, int*  invalid)
 {
    int  res = -1;
 #ifdef __APPLE__
@@ -403,6 +408,7 @@ int  ext_handler_uri(const char*  uri)
     * The variable 'uri' must be prepared so that 'free()' can be called on it
     * at the end.
     */
+   *invalid = 1;
    if(NULL == uri)  { PRINT_ERROR("No URI specified"); }
    else
    {
@@ -421,8 +427,8 @@ int  ext_handler_uri(const char*  uri)
          }
          else
          {
-            printf("%s: %sURI delegation to %s for: %s\n",
-                   CFG_NAME, MAIN_ERR_PREFIX, command, uri);
+            /* URI passed verification */
+            *invalid = 0;
             error = 0;
          }
       }
@@ -431,6 +437,8 @@ int  ext_handler_uri(const char*  uri)
    /* Create command line */
    if(!error)
    {
+      printf("%s: %sURI delegation to %s for: %s\n",
+             CFG_NAME, MAIN_ERR_PREFIX, command, uri);
       len = strlen(command);
       buf = (char*) posix_malloc(++len);  /* One additional byte for NUL */
       if(NULL == buf)  { error = 1; }
