@@ -1400,7 +1400,7 @@ void  filter_exit(void)
                            rv = filter_export_score_rules(fs, fs_tmp);
                         }
                         /* Flush stream of temporary file*/
-                        if (!rv)  { fu_sync(fd_tmp, fs_tmp); }
+                        if (!rv)  { rv = fu_sync(fd_tmp, fs_tmp); }
                         /* Rename temporary file to score file */
                         if(!rv)
                         {
@@ -1408,7 +1408,10 @@ void  filter_exit(void)
                         }
                         if(rv)
                         {
-                           if(tmppathname)  { fu_unlink_file(tmppathname); }
+                           if(tmppathname)
+                           {
+                              (void) fu_unlink_file(tmppathname);
+                           }
                         }
                      }
                      fu_close_file(&fd_tmp, &fs_tmp);
