@@ -907,7 +907,7 @@ int  group_set_list(size_t  num, struct core_groupstate*  grouplist)
    if(!res && num)  { res = group_export_groups(num, grouplist, fs, fs_tmp); }
 
    /* Flush stream of temporary file*/
-   if (!res)  { fu_sync(fd_tmp, fs_tmp); }
+   if(!res)  { res = fu_sync(fd_tmp, fs_tmp); }
 
    /* Rename temporary file to group file */
    if(!res)  { res = posix_rename(tmppathname, grouppathname); }
@@ -916,7 +916,7 @@ int  group_set_list(size_t  num, struct core_groupstate*  grouplist)
    if(res)
    {
       PRINT_ERROR("Failed to store group file");
-      if(tmppathname)  { fu_unlink_file(tmppathname); }
+      if(tmppathname)  { (void) fu_unlink_file(tmppathname); }
    }
 
    /* The tmp file must be removed before releasing the group file lock! */
@@ -1052,7 +1052,7 @@ int  group_sort_list(void)
    posix_free((void*) array);
 
    /* Flush stream of temporary file*/
-   if (!res) { fu_sync(fd_tmp, fs_tmp); }
+   if(!res) { res = fu_sync(fd_tmp, fs_tmp); }
 
    /* Rename temporary file to group file */
    if(!res)  { res = posix_rename(tmppathname, grouppathname); }
@@ -1061,7 +1061,7 @@ int  group_sort_list(void)
    if(res)
    {
       PRINT_ERROR("Failed to store group file");
-      if(tmppathname)  { fu_unlink_file(tmppathname); }
+      if(tmppathname)  { (void) fu_unlink_file(tmppathname); }
    }
 
    /* The tmp file must be removed before releasing the group file lock! */
@@ -1192,7 +1192,7 @@ int  group_add(struct core_groupstate*  group)
    }
 
    /* Flush stream of temporary file*/
-   if (!res) { fu_sync(fd_tmp, fs_tmp); }
+   if(!res) { res = fu_sync(fd_tmp, fs_tmp); }
 
    /* Rename temporary file to group file */
    if(!res)  { res = posix_rename(tmppathname, grouppathname); }
@@ -1201,7 +1201,7 @@ int  group_add(struct core_groupstate*  group)
    if(res)
    {
       PRINT_ERROR("Failed to store group file");
-      if(tmppathname)  { fu_unlink_file(tmppathname); }
+      if(tmppathname)  { (void) fu_unlink_file(tmppathname); }
    }
 
    /* The tmp file must be removed before releasing the group file lock! */
