@@ -33,9 +33,9 @@ extern const char*  main_confprefix;
 /* Function prototypes */
 
 void  print_error(const char*);
+void  ts_environ_init(void);
+void  ts_environ_exit(void);
 int  ts_getenv(const char*, const char**);
-struct tm*  ts_gmtime(const time_t*, struct tm*);
-struct tm*  ts_localtime(const time_t*, struct tm*);
 int  ts_lock_ui(void);
 int  ts_unlock_ui(void);
 
