@@ -345,6 +345,10 @@ long int  posix_random(void);
 #define POSIX_S_IRUSR  S_IRUSR
 #define POSIX_S_IWUSR  S_IWUSR
 #define POSIX_S_IXUSR  S_IXUSR
+#define POSIX_S_IRGRP  S_IRGRP
+#define POSIX_S_IWGRP  S_IWGRP
+#define POSIX_S_IROTH  S_IROTH
+#define POSIX_S_IWOTH  S_IWOTH
 
 #define POSIX_F_GETFL  F_GETFL
 #define POSIX_F_SETFL  F_SETFL
