@@ -260,11 +260,43 @@ int  ts_unlock_ui(void)
 //!
 //! Exported as C style function
 //!
-//! \param[in] msg  String (Error message)
+//! \param[in] msg  String (Unicode error message)
+//!
+//! This function executes as NOP if \e msg is \e NULL .
+//! Otherwise \e msg must be a NUL-terminated string.
+//! <br>
+//! The Unicode encoding of \e msg is checked and is allowed to be invalid.
+//!
+//! \attention
+//! Currently only the US-ASCII subset of Unicode will be readable in output.
+//!
+//! \note
+//! This function is guaranteed to work before other modules are initialized.
 
 void  print_error(const char*  msg)
 {
-   std::clog << CFG_NAME << ": " << msg << std::endl << std::flush;
+   char*  s;
+   std::size_t  len;
+   std::size_t  i;
+   int  c;
+
+   if(NULL != msg)
+   {
+      len = std::strlen(msg);
+      s = new char[len + (std::size_t) 1];
+
+      for(i = 0; i < len; ++i)
+      {
+         c = (int) msg[i];
+         // Accept only printable, HT (9), LF (10, 0x0A) and SPACE (32, 0x20)
+         if(!(9 == c || 10 == c || (32 <= c && 126 >= c)))  { s[i] = '?'; }
+         else  { s[i] = msg[i]; }
+      }
+      s[len] = 0;
+      std::clog << CFG_NAME << ": " << s << std::endl << std::flush;
+
+      delete[] s;
+   }
 }
 
 
