@@ -2,7 +2,7 @@
 /*! \file
  * \brief Transport layer security
  *
- * Copyright (c) 2012-2021 by the developers. See the LICENSE file for details.
+ * Copyright (c) 2012-2022 by the developers. See the LICENSE file for details.
  *
  * If nothing else is specified, function return zero to indicate success
  * and a negative value to indicate an error.
@@ -740,8 +740,7 @@ static int  tls_convert_crl_to_pem(const char*  pn_der, char**  pn_pem)
                {
                   /* CRL stored in PEM format */
                   rv = BIO_get_fd(biop, NULL);
-                  if(-1 != rv)  { fu_sync(rv, NULL); }
-                  res = 0;
+                  if(-1 != rv)  { res = fu_sync(rv, NULL); }
                }
             }
             BIO_free(biop);
@@ -1190,7 +1189,7 @@ static int  tls_chain_check_sigalgs(STACK_OF(X509)*  certificate_chain,
                      " failed");
          break;
       }
-#  if  CFG_USE_OPENSSL_API_1_1
+#  if  CFG_USE_OPENSSL_API_1_1 || CFG_USE_LIBRESSL_API_3_5
       if(EVP_PKEY_RSA != EVP_PKEY_base_id(pkey))
 #  else   /* CFG_USE_OPENSSL_API_1_1 */
       if(EVP_PKEY_RSA != pkey->type)
@@ -1278,8 +1277,9 @@ static int  tls_chain_check_sigalgs(STACK_OF(X509)*  certificate_chain,
 #  if !CFG_USE_OPENSSL_API_3
       RSA_free(rsa_key);
 #  endif  /* !CFG_USE_OPENSSL_API_3 */
+      EVP_PKEY_free(pkey);
       /* Extract hash algorithm for signatures */
-#  if CFG_USE_OPENSSL_API_1_1
+#  if CFG_USE_OPENSSL_API_1_1 || CFG_USE_LIBRESSL_API_3_5
       alg = X509_get_signature_nid(cc);
 #  else   /* CFG_USE_OPENSSL_API_1_1 */
       alg = OBJ_obj2nid(cc->sig_alg->algorithm);
@@ -1739,13 +1739,14 @@ static void  tls_destroy_context(void)
    if(NULL != tls_ctx)
    {
       SSL_CTX_free(tls_ctx);
-      /* Reference count is now zero */
       tls_ctx = NULL;
    }
 
    posix_free((void*) tls_certpath);
+   tls_certpath = NULL;
 #  if !CFG_TLS_CRLS_DISABLE
    posix_free((void*) tls_crlpath);
+   tls_crlpath = NULL;
 #  endif  /* !CFG_TLS_CRLS_DISABLE */
 }
 
@@ -1863,6 +1864,7 @@ int  tls_vulnerability_check(int  check)
    unsigned long int  version = OpenSSL_version_num();
 #  else   /* CFG_USE_OPENSSL_API_1_1 */
 #     if CFG_USE_LIBRESSL && defined(LIBRESSL_VERSION_NUMBER)
+   /* Documentation says LIBRESSL_VERSION_NUMBER always ends with 00F */
    unsigned long int  version = LIBRESSL_VERSION_NUMBER;
 #     else  /* CFG_USE_LIBRESSL && LIBRESSL_VERSION_NUMBER */
    unsigned long int  version = SSLeay();
@@ -1872,10 +1874,10 @@ int  tls_vulnerability_check(int  check)
    unsigned int  minor = (unsigned int) ((version & 0x0FF00000UL) >> 20);
    unsigned int  fix   = (unsigned int) ((version & 0x000FF000UL) >> 12);
    unsigned int  patch = (unsigned int) ((version & 0x00000FF0UL) >> 4);
+#  if ! CFG_USE_LIBRESSL && ! CFG_USE_OPENSSL_API_3
    char  patch_letter;
    char  patch_string[4] = { 0, 0, 0, 0 };
 
-   /* Print OpenSSL version */
    if(patch)
    {
       if(25U >= patch)
@@ -1908,13 +1910,23 @@ int  tls_vulnerability_check(int  check)
       }
       patch_string[0] = patch_letter;
    }
+#  endif  /* ! CFG_USE_LIBRESSL && ! CFG_USE_OPENSSL_API_3 */
+
+   /* Print version */
 #  if CFG_USE_LIBRESSL && defined(LIBRESSL_VERSION_NUMBER)
+   /* LibreSSL */
    printf("%s: %sCompiled for LibreSSL\n", CFG_NAME, MAIN_ERR_PREFIX);
-   printf("%s: %sLibreSSL library version: %X.%X.%X%s (0x%08lX)\n",
-          CFG_NAME, MAIN_ERR_PREFIX, major, minor, fix, patch_string, version);
+   printf("%s: %sLibreSSL library version: %X.%X.%X (0x%08lX)\n",
+          CFG_NAME, MAIN_ERR_PREFIX, major, minor, fix, version);
 #  else   /* CFG_USE_LIBRESSL && LIBRESSL_VERSION_NUMBER */
+   /* OpenSSL */
+#     if CFG_USE_OPENSSL_API_3
+   printf("%s: %sOpenSSL library version: %X.%X.%X (0x%08lX)\n",
+          CFG_NAME, MAIN_ERR_PREFIX, major, minor, patch, version);
+#     else  /* CFG_USE_OPENSSL_API_3 */
    printf("%s: %sOpenSSL library version: %X.%X.%X%s (0x%08lX)\n",
           CFG_NAME, MAIN_ERR_PREFIX, major, minor, fix, patch_string, version);
+#     endif  /* CFG_USE_OPENSSL_API_3 */
 #  endif  /* CFG_USE_LIBRESSL && LIBRESSL_VERSION_NUMBER */
 
    /* Print message if TLSv1.3 is avialable */
@@ -2075,7 +2087,7 @@ int  tls_open(int  sd, void**  co, int  weak, const char*  sni)
    const char*  kx = NULL;
    char  b[200];
 #  if CFG_USE_OPENSSL_API_1_1
-   EVP_PKEY*  key;
+   EVP_PKEY*  key = NULL;
    int  length;
 #  endif  /* CFG_USE_OPENSSL_API_1_1 */
 #  if CFG_USE_OPENSSL_API_1_1
@@ -2218,7 +2230,7 @@ int  tls_open(int  sd, void**  co, int  weak, const char*  sni)
                }
             }
 #  if CFG_USE_OPENSSL_API_1_1
-            if(!weak)
+            if(!res && !weak)
             {
                /* Check FFDHE group (no negotiation possible before TLSv1.3) */
                rv = SSL_get_server_tmp_key(*ssl, &key);
@@ -2247,6 +2259,8 @@ int  tls_open(int  sd, void**  co, int  weak, const char*  sni)
                         }
                      }
                   }
+                  EVP_PKEY_free(key);
+                  key = NULL;
                }
                /* Close connection on error */
                if(res)
@@ -2256,16 +2270,19 @@ int  tls_open(int  sd, void**  co, int  weak, const char*  sni)
                }
             }
 #  else  /* CFG_USE_OPENSSL_API_1_1 */
-#     if CFG_USE_LIBRESSL && (LIBRESSL_VERSION_NUMBER >= 0x30101000L)
-            /* LibreSSL supports TLSv1.3 since version 3.1.1, but no FFDHE */
-            if(strcmp("TLSv1.3", tlsv))
+            if(!res)
             {
-#     endif   /* CFG_USE_LIBRESSL && (LIBRESSL_VERSION_NUMBER >= 0x30101000L) */
-               printf("%s: %sWarning: FFDHE group size check not possible "
-                      "with OpenSSL API 1.0\n", CFG_NAME, MAIN_ERR_PREFIX);
 #     if CFG_USE_LIBRESSL && (LIBRESSL_VERSION_NUMBER >= 0x30101000L)
-            }
+               /* LibreSSL supports TLSv1.3 since version 3.1.1, but no FFDHE */
+               if(strcmp("TLSv1.3", tlsv))
+               {
 #     endif   /* CFG_USE_LIBRESSL && (LIBRESSL_VERSION_NUMBER >= 0x30101000L) */
+                  printf("%s: %sWarning: FFDHE group size check not possible "
+                         "with OpenSSL API 1.0\n", CFG_NAME, MAIN_ERR_PREFIX);
+#     if CFG_USE_LIBRESSL && (LIBRESSL_VERSION_NUMBER >= 0x30101000L)
+               }
+#     endif   /* CFG_USE_LIBRESSL && (LIBRESSL_VERSION_NUMBER >= 0x30101000L) */
+            }
 #  endif  /* CFG_USE_OPENSSL_API_1_1 */
          }
       }
@@ -2293,20 +2310,25 @@ int  tls_open(int  sd, void**  co, int  weak, const char*  sni)
 
 int  tls_close(void**  co)
 {
-   int  res;
+   int  res = -1;
 
-   /* Close connection */
-   res = SSL_shutdown(*((SSL**) co));
-   if(!res)  { res = SSL_shutdown(*((SSL**) co)); }
-
-   /* Destroy context (including buffered CRLs) */
-   tls_destroy_context();
-
-   /* Check for error */
-   if(0 <= res && 1 != res)
+   if(NULL != co && NULL != *((SSL**) co))
    {
-      PRINT_ERROR("Error while closing TLS connection");
-      res = -1;
+      /* Close connection */
+      res = SSL_shutdown(*((SSL**) co));
+      if(!res)  { res = SSL_shutdown(*((SSL**) co)); }
+      if(0 <= res && 1 != res)
+      {
+         PRINT_ERROR("Error while closing TLS connection");
+         res = -1;
+      }
+
+      /* Destroy connection object */
+      SSL_free(*((SSL**) co));
+      *((SSL**) co) = NULL;
+
+      /* Destroy context (including buffered CRLs) */
+      tls_destroy_context();
    }
 
    return(res);
@@ -2472,17 +2494,6 @@ int  tls_cert_verify(void**  co, void**  cert, const char*  cn, int  weak)
       PRINT_ERROR("Extracting certificate chain failed");
       res = -1;
    }
-#  if CFG_USE_OPENSSL_API_1_1 || OPENSSL_VERSION_NUMBER >= 0x10002000L
-   else
-   {
-      certificate_chain = X509_chain_up_ref(certificate_chain);
-      if(NULL == certificate_chain)
-      {
-         PRINT_ERROR("Duplicating certificate chain failed");
-         res = -1;
-      }
-   }
-#  endif  /* CFG_USE_OPENSSL_API_1_1 || (OPENSSL_VERSION_NUMBER >= 1.0.2) */
    if(!res && weak)
    {
       printf("%s: %sWarning: Using weak certificate chain checks\n",
@@ -2539,12 +2550,6 @@ int  tls_cert_verify(void**  co, void**  cert, const char*  cn, int  weak)
       }
    }
 
-   /* Destroy certificate chain */
-   if(NULL != certificate_chain)
-   {
-      sk_X509_pop_free(certificate_chain, X509_free);
-   }
-
    /* Verify server certificate */
    if(!res)
    {
@@ -2752,7 +2757,10 @@ posix_ssize_t  tls_send(void*  co, const void*  buf, size_t  len)
    int  num;
 
    /* Check size */
-   if(INT_MAX < len)  { PRINT_ERROR("tls_send(): Data length too large"); }
+   if(POSIX_INT_MAX < len)
+   {
+      PRINT_ERROR("tls_send(): Data length too large");
+   }
    num = (int) len;
 
    res = (posix_ssize_t) SSL_write((SSL*) co, buf, num);
@@ -2786,7 +2794,10 @@ posix_ssize_t  tls_recv(void*  co, void*  buf, size_t  len, int  peek)
    int  num;
 
    /* Check size */
-   if(INT_MAX < len)  { PRINT_ERROR("tls_recv(): Data length too large"); }
+   if(POSIX_INT_MAX < len)
+   {
+      PRINT_ERROR("tls_recv(): Data length too large");
+   }
    num = (int) len;
 
    /* Check for peek flag */
